require 'uri'
require 'screencap'

class ThumbnailGenerateWorker
  include Sidekiq::Worker

  def perform(media_path, object_id)
    media = create_media media_path
    thumbnail_service.update_object(thumbnail_service.create_thumbnail(media), object_id)
  end

  private

  def create_media(path)
    return ::Screencap::Fetcher.new(path) if url =~ URI::regexp
    File.open path
  end

  def thumbnail_service
    @@thumbnailService ||= ThumbnailService.new(generators)
  end

  def generators
    [
        Thumbnail::Strategies::VideoThumbnailGenerator.new,
        Thumbnail::Strategies::ImageThumbnailGenerator.new,
        Thumbnail::Strategies::PdfThumbnailGenerator.new,
        Thumbnail::Strategies::UrlThumbnailGenerator.new
    ]
  end
end
